@@ -1,21 +1,31 @@
 # This file is part of Xpra.
-# Copyright (C) 2013-2021 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2013-2023 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os.path
 
 from xpra.net.digest import get_salt, choose_digest
-from xpra.os_util import strtobytes
 from xpra.server.auth.sys_auth_base import SysAuthenticator
 from xpra.log import Logger
 
 log = Logger("auth")
 
 
+def stat_filetime(full_path) -> float:
+    try:
+        v = os.stat(full_path).st_mtime
+        log("mtime(%s)=%s", full_path, v)
+        return v
+    except Exception as e:
+        log.error(f"Error accessing time of file {full_path!r}")
+        log.estr(e)
+        return 0
+
+
 class FileAuthenticatorBase(SysAuthenticator):
     def __init__(self, **kwargs):
-        password_file = kwargs.pop("filename", None)
+        password_file = kwargs.pop("filename", "")
         log("FileAuthenticatorBase password_file=%s", password_file)
         if not password_file:
             log.warn("Warning: %r authentication module is missing the 'filename' option", self)
@@ -25,15 +35,18 @@ class FileAuthenticatorBase(SysAuthenticator):
             password_file = os.path.join(exec_cwd, password_file)
         log("FileAuthenticatorBase filename=%s", password_file)
         super().__init__(**kwargs)
+        self.salt: bytes | bool | None = None
+        self.digest: str = ""
+        self.challenge_sent = False
         self.password_filename = password_file
         self.password_filedata = None
-        self.password_filetime = None
+        self.password_filetime = 0.0
         self.authenticate_check = self.authenticate_hmac
 
     def requires_challenge(self) -> bool:
         return True
 
-    def get_challenge(self, digests):
+    def get_challenge(self, digests: tuple[str, ...]) -> tuple[bytes, str] | None:
         if self.salt is not None:
             log.error("challenge already sent!")
             if self.salt is not False:
@@ -46,14 +59,14 @@ class FileAuthenticatorBase(SysAuthenticator):
 
     def get_password(self) -> str:
         file_data = self.load_password_file()
-        if file_data is None:
-            return None
-        return strtobytes(file_data)
+        if not file_data:
+            return ""
+        return file_data
 
-    def parse_filedata(self, data):
-        return data
+    def parse_filedata(self, data: str):  # pragma: no cover
+        raise NotImplementedError()
 
-    def load_password_file(self) -> bytes:
+    def load_password_file(self):
         if not self.password_filename:
             return None
         full_path = os.path.abspath(self.password_filename)
@@ -62,28 +75,21 @@ class FileAuthenticatorBase(SysAuthenticator):
             self.password_filedata = None
         else:
             ptime = self.stat_password_filetime()
-            if self.password_filedata is None or ptime!=self.password_filetime:
-                self.password_filetime = None
-                self.password_filedata = None
+            if self.password_filedata is None or ptime != self.password_filetime:
+                self.password_filetime = 0
+                self.password_filedata = b""
                 try:
-                    with open(self.password_filename, mode="rb") as f:
+                    with open(self.password_filename) as f:
                         data = f.read()
                     log(f"loaded {len(data)} bytes from {self.password_filename!r}")
                     self.password_filedata = self.parse_filedata(data)
                     self.password_filetime = ptime
                 except Exception as e:
                     log.error("Error reading password data from '%s':", self.password_filename, exc_info=True)
-                    log.error(" %s", e)
+                    log.estr(e)
                     self.password_filedata = None
         return self.password_filedata
 
-    def stat_password_filetime(self) -> int:
-        try:
-            full_path = os.path.abspath(self.password_filename)
-            v = os.stat(full_path).st_mtime
-            log("mtime(%s)=%s", full_path, v)
-            return v
-        except Exception as e:
-            log.error("Error accessing time of password file '%s'", full_path)
-            log.error(" %s", e)
-            return 0
+    def stat_password_filetime(self) -> float:
+        full_path = os.path.abspath(self.password_filename or "")
+        return stat_filetime(full_path)
